/***********************************************************************
* FileName: humidity_fan.c
* Dependencies: none
* Processor: Microchip PIC
* Compiler: CCS v3.249
*
* SOFTWARE IS PROVIDED "AS IS." DIGI-KEY EXPRESSLY DISCLAIMS ANY
* WARRANTY OF ANY KIND, WHETHER EXPRESS OR IMPLIED, INCLUDING BUT NOT
* LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
* PARTICULAR PURPOSE, OR NON-INFRINGEMENT. IN NO EVENT SHALL DIGI-KEY
* BE LIABLE FOR ANY INCIDENTAL, SPECIAL, INDIRECT OR CONSEQUENTIAL
* DAMAGES, LOST PROFITS OR LOST DATA, HARM TO YOUR EQUIPMENT, COST OF
* PROCUREMENT OF SUBSTITUTE GOODS, TECHNOLOGY OR SERVICES, ANY CLAIMS
* BY THIRD PARTIES (INCLUDING BUT NOT LIMITED TO ANY DEFENSE THEREOF),
* ANY CLAIMS FOR INDEMNITY OR CONTRIBUTION, OR OTHER SIMILAR COSTS.
* DIGI-KEY ALSO DISCLAIMS ANY LIABILITY FOR PATENT OR COPYRIGHT
* INFRINGEMENT.
*
* Version History
* Version 1.0 08/04/06 J Purcell
* Version 1.2 09/01/06 J Purcell
* Initial Public Release
* Version 1.2 03/01/08 
**********************************************************************/

/*
* Program used to check the humidity of surrounding space, if the
* humidity is too low then a motor will kick on and humidify the space.
* This could be used with the opposite intentions(to dry an area if
* humidity is too high)				
*/

#include <12F683.h>     //this file comes with PICC C Compiler
#include <my12F683.h>   //file I created for simplified programming

//Declarations

void Wait_4ms(unsigned int);
void Init(void);

//Global Variables

int REAL_Humidity;
int MIN_Humidity;
int BIG_COUNT;

/***********************************************************************
*
* Function Name: Wait_4ms
*
* Returns: None
*
* Parameters: unsigned int
*
* Purpose: Delay for timing sequences
* at X=250 you have one second time delay   if clk=4Mhz
**********************************************************************/

void Wait_4ms(unsigned int X)
{
   unsigned int Y;
   unsigned int P;
   do
   {
      Y = 4;
      do
      {
         P = 166;
         do
         {
            P -= 1;
         } while (P > 0);
         Y -= 1;
      } while (Y > 0);
      X -= 1;
   } while (X > 0);
}

/***********************************************************************
*
* Function Name: Init
*
* Returns: None
*
* Parameters: None
*
* Purpose: MCU setup and Initialization
*
**********************************************************************/

void Init(void)
{
   PIE1=0x01;
   T1CON=0x06;         //timer for internal clock and sleep operations
   OPTION_REG=0x64;    //setup of sleep mode and timer0 prescale
   ANSEL=0x00;         //all digital channels
   PEIE=1;
   GPIO = 0x03;
   TRISA=0xAC;
   WPU=0xAC;
}

/***********************************************************************
*
* Function Name: main
*
* Returns: None
*
* Parameters: None
*
* Purpose: humidity testing & sleep
* The frequency for testing is determined by taking TMR0 and
* multiplying by 32 (the prescaler)
**********************************************************************/

void main (void)
{

   Init();
   MIN_Humidity = 204;  //min humidity ~64%  changes with each circuit
   BIG_COUNT=0x0000;
   do
   {
      Wait_4ms(2);      //a short wait for stablization
      RA0=0;            // turn on led 
      TMR0=0x00;        //clear timer for counting
      Wait_4ms(250);    //led shows the data collection time ~1sec
      BIG_COUNT=0x0000;
      BIG_COUNT=TMR0;   //store the number of edges
      RA0=1;            //turn off LED
                        //testing the count for low humidity
      if (BIG_COUNT > (MIN_Humidity ))   
      {
         RA4=1;         //motor on
      }
      if (BIG_COUNT < (MIN_Humidity ))
      {
         RA4=0;         //motor off
      }

                        //Timer1 setup for sleep

      TMR1H=0x00;       //these are precautionary
      TMR1L=0x00;
      TMR1IF=0;         //clear flag
      RA1 = 0;          //LED signal for sleep 
      Wait_4ms(4);      //wait is for visual verification
      RA1 = 1;          //LED off
      TMR1ON=1;         //timer 1 on for wake-up
      SLEEP();          //Compliler specific call
      TMR1ON=0;         //timer 1 off while in main loop

   } while(1>0);        //end do while loop

}                       //end of main program